namespace sGUI

declare function NewMenuBase as sGUIList ptr
declare function OpenMenuStrip(menulist as sGUIList ptr, txt as string) as integer
declare function CloseMenuStrip(menulist as sGUIList ptr) as integer
declare function Item(menulist as sGUIList ptr, txt as string, activation as integer=1) as integer
declare function CheckMark(menulist as sGUIList ptr, txt as string, activation as integer=1, selection as integer) as integer
declare function Separator(menulist as sGUIList ptr) as integer
declare function MenuValidator(menulist as sGUIList ptr) as integer
declare sub MenuSetLevel(menulist as sGUIList ptr)
declare function MenuSetGroup(menulist as sGUIList ptr) as integer
declare sub MenuResizeAll(menulist as sGUIList ptr, mode as integer=0)
declare sub ProofPrint (menulist as sGUIList ptr)

function NewMenuBase as sGUIList ptr
  function=cast( sGUIList ptr, EntryLists->AppendNew(sGUIListType) )
end function


function OpenMenuStrip(menulist as sGUIList ptr, txt as string) as integer
  function=0
  dim as sGUIEntry ptr entry
  dim as integer level  
  entry=menulist->AppendNew(sGUIEntryType)
  
  if entry then
    entry->Text         =txt 
    entry->EntryType    =ETYPE_OPENMENUSTRIP
    entry->EntryWidth   =GetTextWidth(entry->Text)+2*LeastGap 
    entry->EntryHeight  =GetFontHeight+2*LeastGap
    function=entry->index
  end if
end function


function CloseMenuStrip(menulist as sGUIList ptr) as integer
  function=0
  dim as sGUIEntry ptr entry
  dim as integer level  
  entry=menulist->AppendNew(sGUIEntryType)
  
  if entry then
    entry->EntryType      =ETYPE_CLOSEMENUSTRIP
    entry->EntryWidth     =0
    entry->EntryHeight    =0
		function=entry->index
  end if
end function


function Item(menulist as sGUIList ptr, txt as string, activation as integer=1) as integer
  function=0
  dim as sGUIEntry ptr entry
  dim as integer level  
  entry=menulist->AppendNew(sGUIEntryType)
  
  if entry then
    entry->Text         =txt 
    entry->EntryType    =ETYPE_ITEM
    entry->Activation   =activation
    entry->EntryWidth   =GetTextWidth(entry->Text)+2*LeastGap 
    entry->EntryHeight  =GetFontHeight+2*LeastGap   
    function=entry->index
  end if 
end function


function CheckMark(menulist as sGUIList ptr, txt as string, activation as integer=1, selection as integer) as integer
  function=0
  dim as sGUIEntry ptr entry
  dim as integer level  
  entry=menulist->AppendNew(sGUIEntryType)
  
  if entry then
    entry->Text           =txt 
    entry->EntryType      =ETYPE_CHECKMARK
    entry->Activation     =activation
    entry->Selection      =selection
    entry->EntryWidth     =GetTextWidth(entry->Text)+2*LeastGap 
    entry->EntryHeight    =GetFontHeight+2*LeastGap    
    function=entry->index 
  end if  
end function


function Separator(menulist as sGUIList ptr) as integer
  function=0
  dim as sGUIEntry ptr entry
  dim as integer level  
  entry=menulist->AppendNew(sGUIEntryType)
  
  if entry then
    entry->EntryType      =ETYPE_SEPARATOR
    entry->Activation     =0
    entry->EntryWidth     =0
    entry->EntryHeight    =2 + 2*LeastGap    
    function=entry->index
  end if
end function

'1.) stack mu zum Schlu 0 sein, d.h. gleiche Anzahl Open und Close
'2.) wenn stack=0 darf nur ein Open erfolgen, schliet alle(!) anderen Eintrge aus 
'3.) stack darf zur Laufzeit des Checks nicht unter 0 fallen, wre bei mehr Close als Open der Fall -> Fehler!!
'4.) auf ein Open darf nicht direkt ein Close folgen -> dies wre ein leerer Menstrip -> unzulssig
function MenuValidator(menulist as sGUIList ptr) as integer
  dim as integer stack,fail,lasttype
  dim as sGUIEntry ptr entry
  stack=0    
  fail=0
  lasttype=-1
  entry=menulist->GetFirst
  if entry then
    do   
      if (stack=0) and (entry->EntryType <> ETYPE_OPENMENUSTRIP) then fail+=1'siehe 2.) und 3.) oben
      
      if entry->EntryType=ETYPE_OPENMENUSTRIP then stack +=1
      if entry->EntryType=ETYPE_CLOSEMENUSTRIP then 
        stack -=1
        if lasttype=ETYPE_OPENMENUSTRIP then fail+=1'siehe 4.) oben
      end if
      
      lasttype=entry->EntryType
      entry=cast(sGUIEntry ptr,entry->next_node)'nchsten Eintrag holen       
    loop until (entry=0) or (fail>0)
    
    if stack>0 then fail+=1'siehe 1.) oben  
  end if

  if fail=0 then function=1 else function=0
end function


sub MenuSetLevel(menulist as sGUIList ptr)
  dim as sGUIEntry ptr entry=menulist->GetFirst
  dim as integer lvl=0
  
  if entry then
    do
      'die Reihenfolge von Erhhen/Verringern/Zuordnen bestimmt,
      'ob ein CloseMenuStrip auf OpenMenuStrip-Level erscheint oder im Strip-Level als
      'eine Art "EndMarker".
      
      'dies hier ist Variante CloseMenuStrip als Strip-EndMarker:     
      entry->Level=lvl                                      'Level Zuordnen
      if entry->EntryType=ETYPE_CLOSEMENUSTRIP then lvl -=1 'Level Verringern   
      if entry->EntryType=ETYPE_OPENMENUSTRIP then lvl +=1  'Level Erhhen   
      
      entry=cast(sGUIEntry ptr,entry->next_node)'nchsten Eintrag holen       
    loop until entry=0
  end if
end sub


function MenuSetGroup(menulist as sGUIList ptr) as integer
  dim as sGUIEntry ptr entry,e
  dim as integer lvl,up,group
  entry=menulist->GetFirst  
  lvl=0
  group=0
  if entry then
    do
      up=0
      e=entry
      do
        if e->Level>lvl then up=1
        
        if e->Level=lvl then
          e->MemberOfGroup=Group
          if e->EntryType=ETYPE_CLOSEMENUSTRIP then group +=1'aktuelle Gruppe schliessen und eine neu Gruppe "ffnen"
        end if
               
        e=cast(sGUIEntry ptr,e->next_node)'nchsten Eintrag holen       
      loop until e=0
      
      'da oben (in MenuSetLevel()) der CloseMenuStrip als Strip-EndMarker definiert ist,
      'gibt es auf Level 0 kein CloseMenuStrip der die aktuelle Gruppe schliessen und eine neu Gruppe "ffnen" wrde,
      'darum muss das fr Level 0 von Hand geschehen:
      if lvl=0 then group +=1
            
      if up=1 then lvl +=1  
    loop until up=0
  end if
  function=group
end function


sub MenuResizeAll(menulist as sGUIList ptr, mode as integer=0)
  dim as sGUIEntry ptr entry,e
  dim as integer group,up,tempWidth
  entry=menulist->GetFirst  
  group=0
  if entry then
    do
      up=0
      e=entry
      tempWidth=0
      'erster Lauf, breitesten Eintrag der Gruppe finden
      do
        if e->MemberOfGroup>group then up=1
        
        if e->MemberOfGroup=group then
          if e->EntryWidth>tempWidth then tempWidth=e->EntryWidth
        end if
               
        e=cast(sGUIEntry ptr,e->next_node)'nchsten Eintrag holen       
      loop until e=0
          
      'zweiter Lauf, in allen Gruppenmitgliedern die neue Gre festlegen 
      e=entry
      do
        
        if (e->MemberOfGroup=group) and (e->EntryType<>2) then
          
          if group=0 then
            'mode 0 MenuBar Fenster: alle Eintrge behalten ihre individuelle Breite, horizontal angeordnet, dann passiert hier nichts
            'mode 1 Popup Men: alle Eintrge erhalten gleiche Breite, vertikal angeordnet 
            if mode=1 then e->EntryWidth=tempWidth          
          else
            e->EntryWidth=tempWidth +32'bei allen anderen Gruppen Breite anpassen          
          end if

        end if
               
        e=cast(sGUIEntry ptr,e->next_node)'nchsten Eintrag holen       
      loop until e=0 
      
      if up=1 then group +=1  
    loop until up=0
  end if
end sub


sub ProofPrint (menulist as sGUIList ptr)
  dim as sGUIEntry ptr entry
  dim as string et,sp

  print tab(15);"L / G"
  entry=menulist->GetFirst
  if entry then
    do
      sp=space(entry->Level*2)
      et=""
      select case entry->EntryType
        case ETYPE_OPENMENUSTRIP
          et="OPEN"
        case ETYPE_CLOSEMENUSTRIP
          et="CLOS"
        case ETYPE_ITEM
          et="ITEM"
        case ETYPE_CHECKMARK
          et="CMRK"
        case ETYPE_SEPARATOR
          et="SEPR"
      end select
      if et<>"" then

        print tab(1 + entry->Level*2); et; tab(15); entry->Level & " / " & entry->MemberOfGroup, entry->EntryWidth & "/" & entry->EntryHeight
      end if
      entry=cast(sGUIEntry ptr,entry->next_node)'nchsten Eintrag holen       
    loop until entry=0
  end if    
end sub



end namespace
